@@ -1,12 +1,12 @@
 from __future__ import annotations
 
-import contextlib
 import csv
+import functools
 import itertools
 import json
-import os
 import threading
 
+from collections import defaultdict
 from concurrent.futures import ThreadPoolExecutor
 from concurrent.futures import wait
 from pathlib import Path
@@ -14,11 +14,10 @@ from subprocess import CalledProcessError
 from typing import TYPE_CHECKING
 from typing import Any
 
-from cleo.io.null_io import NullIO
 from poetry.core.packages.utils.link import Link
 
+from poetry.console.exceptions import PoetryRuntimeError
 from poetry.installation.chef import Chef
-from poetry.installation.chef import ChefBuildError
 from poetry.installation.chooser import Chooser
 from poetry.installation.operations import Install
 from poetry.installation.operations import Uninstall
@@ -28,17 +27,23 @@ from poetry.puzzle.exceptions import SolverProblemError
 from poetry.utils._compat import decode
 from poetry.utils.authenticator import Authenticator
 from poetry.utils.env import EnvCommandError
-from poetry.utils.helpers import atomic_open
+from poetry.utils.helpers import Downloader
 from poetry.utils.helpers import get_file_hash
+from poetry.utils.helpers import get_highest_priority_hash_type
 from poetry.utils.helpers import pluralize
 from poetry.utils.helpers import remove_directory
-from poetry.utils.pip import pip_install
+from poetry.utils.isolated_build import IsolatedBuildBackendError
+from poetry.utils.isolated_build import IsolatedBuildInstallError
+from poetry.vcs.git import Git
 
 
 if TYPE_CHECKING:
+    from collections.abc import Mapping
+    from collections.abc import Sequence
+
     from cleo.io.io import IO
     from cleo.io.outputs.section_output import SectionOutput
-    from poetry.core.masonry.builders.builder import Builder
+    from packaging.utils import NormalizedName
     from poetry.core.packages.package import Package
 
     from poetry.config.config import Config
@@ -47,6 +52,12 @@ if TYPE_CHECKING:
     from poetry.utils.env import Env
 
 
+def _package_get_name(package: Package) -> str | None:
+    if url := package.repository_url:
+        return Git.get_name_from_source_url(url)
+    return None
+
+
 class Executor:
     def __init__(
         self,
@@ -63,29 +74,23 @@ class Executor:
         self._enabled = True
         self._verbose = False
         self._wheel_installer = WheelInstaller(self._env)
-        self._use_modern_installation = config.get(
-            "installer.modern-installation", True
-        )
 
         if parallel is None:
             parallel = config.get("installer.parallel", True)
 
         if parallel:
-            self._max_workers = self._get_max_workers(
-                desired_max_workers=config.get("installer.max-workers")
-            )
+            self._max_workers = config.installer_max_workers
         else:
             self._max_workers = 1
 
+        self._artifact_cache = pool.artifact_cache
         self._authenticator = Authenticator(
             config, self._io, disable_cache=disable_cache, pool_size=self._max_workers
         )
-        self._chef = Chef(config, self._env, pool)
+        self._chef = Chef(self._artifact_cache, self._env, pool)
         self._chooser = Chooser(pool, self._env, config)
 
         self._executor = ThreadPoolExecutor(max_workers=self._max_workers)
-        self._total_operations = 0
-        self._executed_operations = 0
         self._executed = {"install": 0, "update": 0, "uninstall": 0}
         self._skipped = {"install": 0, "update": 0, "uninstall": 0}
         self._sections: dict[int, SectionOutput] = {}
@@ -94,6 +99,16 @@ class Executor:
         self._shutdown = False
         self._hashes: dict[str, str] = {}
 
+        # Cache whether decorated output is supported.
+        # https://github.com/python-poetry/cleo/issues/423
+        self._decorated_output: bool = self._io.output.is_decorated()
+        self._max_retries = config.get("requests.max-retries", 0)
+
+        # sdist build config settings
+        self._build_config_settings: Mapping[
+            NormalizedName, Mapping[str, str | Sequence[str]]
+        ] = config.get("installer.build-config-settings")
+
     @property
     def installations_count(self) -> int:
         return self._executed["install"]
@@ -106,8 +121,12 @@ class Executor:
     def removals_count(self) -> int:
         return self._executed["uninstall"]
 
+    @property
+    def enabled(self) -> bool:
+        return self._enabled
+
     def supports_fancy_output(self) -> bool:
-        return self._io.output.is_decorated() and not self._dry_run
+        return self._decorated_output and not self._dry_run
 
     def disable(self) -> Executor:
         self._enabled = False
@@ -127,24 +146,7 @@ class Executor:
     def enable_bytecode_compilation(self, enable: bool = True) -> None:
         self._wheel_installer.enable_bytecode_compilation(enable)
 
-    def pip_install(
-        self, req: Path, upgrade: bool = False, editable: bool = False
-    ) -> int:
-        try:
-            pip_install(req, self._env, upgrade=upgrade, editable=editable)
-        except EnvCommandError as e:
-            output = decode(e.e.output)
-            if (
-                "KeyboardInterrupt" in output
-                or "ERROR: Operation cancelled by user" in output
-            ):
-                return -2
-            raise
-
-        return 0
-
     def execute(self, operations: list[Operation]) -> int:
-        self._total_operations = len(operations)
         for job_type in self._executed:
             self._executed[job_type] = 0
             self._skipped[job_type] = 0
@@ -155,7 +157,8 @@ class Executor:
         self._sections = {}
         self._yanked_warnings = []
 
-        # pip has to be installed first without parallelism if we install via pip
+        # pip has to be installed/updated first without parallelism
+        # because we still need it for uninstalls
         for i, op in enumerate(operations):
             if op.package.name == "pip":
                 wait([self._executor.submit(self._execute_operation, op)])
@@ -167,6 +170,7 @@ class Executor:
         for _, group in groups:
             tasks = []
             serial_operations = []
+            serial_git_operations = defaultdict(list)
             for operation in group:
                 if self._shutdown:
                     break
@@ -181,26 +185,49 @@ class Executor:
                     operation.package.develop
                     and operation.package.source_type in {"directory", "git"}
                 )
-                if not operation.skipped and is_parallel_unsafe:
-                    serial_operations.append(operation)
-                    continue
+                # Skipped operations are safe to execute in parallel
+                if operation.skipped:
+                    is_parallel_unsafe = False
 
-                tasks.append(self._executor.submit(self._execute_operation, operation))
+                if is_parallel_unsafe:
+                    serial_operations.append(operation)
+                elif operation.package.source_type == "git":
+                    # Serially execute git operations that get cloned to the same directory,
+                    # to prevent multiple parallel git operations in the same repo.
+                    serial_git_operations[_package_get_name(operation.package)].append(
+                        operation
+                    )
+                else:
+                    tasks.append(
+                        self._executor.submit(self._execute_operation, operation)
+                    )
+
+            def _serialize(
+                repository_serial_operations: list[Operation],
+            ) -> None:
+                for operation in repository_serial_operations:
+                    self._execute_operation(operation)
+
+            # For each git repository, execute all operations serially
+            for repository_git_operations in serial_git_operations.values():
+                tasks.append(
+                    self._executor.submit(
+                        _serialize,
+                        repository_serial_operations=repository_git_operations,
+                    )
+                )
 
             try:
                 wait(tasks)
 
                 for operation in serial_operations:
-                    wait([self._executor.submit(self._execute_operation, operation)])
+                    self._execute_operation(operation)
 
             except KeyboardInterrupt:
                 self._shutdown = True
 
             if self._shutdown:
-                # Cancelling further tasks from being executed
-                [task.cancel() for task in tasks]
-                self._executor.shutdown(wait=True)
-
+                self._executor.shutdown(wait=True, cancel_futures=True)
                 break
 
         for warning in self._yanked_warnings:
@@ -216,21 +243,6 @@ class Executor:
 
         return 1 if self._shutdown else 0
 
-    @staticmethod
-    def _get_max_workers(desired_max_workers: int | None = None) -> int:
-        # This should be directly handled by ThreadPoolExecutor
-        # however, on some systems the number of CPUs cannot be determined
-        # (it raises a NotImplementedError), so, in this case, we assume
-        # that the system only has one CPU.
-        try:
-            default_max_workers = (os.cpu_count() or 1) + 4
-        except NotImplementedError:
-            default_max_workers = 5
-
-        if desired_max_workers is None:
-            return default_max_workers
-        return min(default_max_workers, desired_max_workers)
-
     def _write(self, operation: Operation, line: str) -> None:
         if not self.supports_fancy_output() or not self._should_write_operation(
             operation
@@ -259,18 +271,18 @@ class Executor:
                     with self._lock:
                         self._sections[id(operation)] = self._io.section()
                         self._sections[id(operation)].write_line(
-                            f"  <fg=blue;options=bold>•</> {op_message}:"
+                            f"  <fg=blue;options=bold>-</> {op_message}:"
                             " <fg=blue>Pending...</>"
                         )
             else:
                 if self._should_write_operation(operation):
                     if not operation.skipped:
                         self._io.write_line(
-                            f"  <fg=blue;options=bold>•</> {op_message}"
+                            f"  <fg=blue;options=bold>-</> {op_message}"
                         )
                     else:
                         self._io.write_line(
-                            f"  <fg=default;options=bold,dark>•</> {op_message}: "
+                            f"  <fg=default;options=bold,dark>-</> {op_message}: "
                             "<fg=default;options=bold,dark>Skipped</> "
                             "<fg=default;options=dark>for the following reason:</> "
                             f"<fg=default;options=bold,dark>{operation.skip_reason}</>"
@@ -289,7 +301,7 @@ class Executor:
             # error to be picked up by the error handler.
             if result == -2:
                 raise KeyboardInterrupt
-        except Exception as e:  # noqa: PIE786
+        except Exception as e:
             try:
                 from cleo.ui.exception_trace import ExceptionTrace
 
@@ -298,7 +310,7 @@ class Executor:
                     io = self._io
                 else:
                     message = (
-                        "  <error>•</error>"
+                        "  <error>-</error>"
                         f" {self.get_operation_message(operation, error=True)}:"
                         " <error>Failed</error>"
                     )
@@ -306,45 +318,79 @@ class Executor:
                     io = self._sections.get(id(operation), self._io)
 
                 with self._lock:
-                    trace = ExceptionTrace(e)
-                    trace.render(io)
-                    if isinstance(e, ChefBuildError):
-                        pkg = operation.package
-                        pip_command = "pip wheel --use-pep517"
+                    pkg = operation.package
+                    with_trace = True
+
+                    if isinstance(e, IsolatedBuildBackendError):
+                        # TODO: Revisit once upstream fix is available https://github.com/python-poetry/cleo/issues/454
+                        # we disable trace here explicitly to workaround incorrect context detection by crashtest
+                        with_trace = False
+                        pip_command = "pip wheel --no-cache-dir --use-pep517"
                         if pkg.develop:
-                            requirement = pkg.source_url
+                            if pkg.source_type == "git":
+                                git_url_parts = (
+                                    pkg.to_dependency()
+                                    .to_pep_508()
+                                    .split(";", 1)[0]
+                                    .split("@", 1)[-1]
+                                    .strip()
+                                ).split("#", 1)
+                                requirement = f"{git_url_parts[0]}#egg={pkg.name}"
+                                if len(git_url_parts) > 1:
+                                    requirement += f"&{git_url_parts[1]}"
+                            else:
+                                assert pkg.source_url
+                                requirement = pkg.source_url
                             pip_command += " --editable"
                         else:
                             requirement = (
                                 pkg.to_dependency().to_pep_508().split(";")[0].strip()
                             )
-                        io.write_line("")
-                        io.write_line(
-                            "<info>"
-                            "Note: This error originates from the build backend,"
-                            " and is likely not a problem with poetry"
-                            f" but with {pkg.pretty_name} ({pkg.full_pretty_version})"
-                            " not supporting PEP 517 builds. You can verify this by"
-                            f" running '{pip_command} \"{requirement}\"'."
-                            "</info>"
+
+                        if config_settings := self._build_config_settings.get(pkg.name):
+                            for setting in config_settings:
+                                for setting_value in config_settings[setting]:
+                                    pip_command += f" --config-settings='{setting}={setting_value}'"
+
+                        message = e.generate_message(
+                            source_string=f"{pkg.pretty_name} ({pkg.full_pretty_version})",
+                            build_command=f'{pip_command} "{requirement}"',
+                        )
+                    elif isinstance(e, IsolatedBuildInstallError):
+                        message = (
+                            "<error>"
+                            "Cannot install build-system.requires"
+                            f" for {pkg.pretty_name}."
+                            "</error>"
                         )
                     elif isinstance(e, SolverProblemError):
-                        pkg = operation.package
-                        io.write_line("")
-                        io.write_line(
+                        message = (
                             "<error>"
                             "Cannot resolve build-system.requires"
                             f" for {pkg.pretty_name}."
                             "</error>"
                         )
+                    elif isinstance(e, PoetryRuntimeError):
+                        message = e.get_text(io.is_verbose(), indent="    | ").rstrip()
+                        message = f"<warning>{message}</>"
+                        with_trace = False
+                    else:
+                        message = f"<error>Cannot install {pkg.pretty_name}.</error>"
+
+                    if with_trace:
+                        ExceptionTrace(e).render(io)
+
+                    io.write_line("")
+                    io.write_line(message)
                     io.write_line("")
             finally:
                 with self._lock:
                     self._shutdown = True
+
         except KeyboardInterrupt:
             try:
                 message = (
-                    "  <warning>•</warning>"
+                    "  <warning>-</warning>"
                     f" {self.get_operation_message(operation, warning=True)}:"
                     " <warning>Cancelled</warning>"
                 )
@@ -364,12 +410,10 @@ class Executor:
             if self.supports_fancy_output():
                 self._write(
                     operation,
-                    (
-                        f"  <fg=default;options=bold,dark>•</> {operation_message}: "
-                        "<fg=default;options=bold,dark>Skipped</> "
-                        "<fg=default;options=dark>for the following reason:</> "
-                        f"<fg=default;options=bold,dark>{operation.skip_reason}</>"
-                    ),
+                    f"  <fg=default;options=bold,dark>-</> {operation_message}: "
+                    "<fg=default;options=bold,dark>Skipped</> "
+                    "<fg=default;options=dark>for the following reason:</> "
+                    f"<fg=default;options=bold,dark>{operation.skip_reason}</>",
                 )
 
             self._skipped[operation.job_type] += 1
@@ -385,7 +429,7 @@ class Executor:
             return result
 
         operation_message = self.get_operation_message(operation, done=True)
-        message = f"  <fg=green;options=bold>•</> {operation_message}"
+        message = f"  <fg=green;options=bold>-</> {operation_message}"
         self._write(operation, message)
 
         self._increment_operations_count(operation, True)
@@ -395,7 +439,6 @@ class Executor:
     def _increment_operations_count(self, operation: Operation, executed: bool) -> None:
         with self._lock:
             if executed:
-                self._executed_operations += 1
                 self._executed[operation.job_type] += 1
             else:
                 self._skipped[operation.job_type] += 1
@@ -455,13 +498,18 @@ class Executor:
             )
 
         if isinstance(operation, Update):
+            initial_version = (initial_pkg := operation.initial_package).version
+            target_version = (target_pkg := operation.target_package).version
+            update_kind = (
+                "Updating" if target_version >= initial_version else "Downgrading"
+            )
             return (
-                f"<{base_tag}>Updating"
-                f" <{package_color}>{operation.initial_package.name}</{package_color}> "
+                f"<{base_tag}>{update_kind}"
+                f" <{package_color}>{initial_pkg.name}</{package_color}> "
                 f"(<{source_operation_color}>"
-                f"{operation.initial_package.full_pretty_version}"
+                f"{initial_pkg.full_pretty_version}"
                 f"</{source_operation_color}> -> <{operation_color}>"
-                f"{operation.target_package.full_pretty_version}</>)</>"
+                f"{target_pkg.full_pretty_version}</>)</>"
             )
         return ""
 
@@ -514,20 +562,18 @@ class Executor:
 
     def _execute_uninstall(self, operation: Uninstall) -> int:
         op_msg = self.get_operation_message(operation)
-        message = f"  <fg=blue;options=bold>•</> {op_msg}: <info>Removing...</info>"
+        message = f"  <fg=blue;options=bold>-</> {op_msg}: <info>Removing...</info>"
         self._write(operation, message)
 
         return self._remove(operation.package)
 
     def _install(self, operation: Install | Update) -> int:
         package = operation.package
-        if package.source_type == "directory" and not self._use_modern_installation:
-            return self._install_directory_without_wheel_installer(operation)
 
         cleanup_archive: bool = False
         if package.source_type == "git":
             archive = self._prepare_git_archive(operation)
-            cleanup_archive = True
+            cleanup_archive = operation.package.develop
         elif package.source_type == "file":
             archive = self._prepare_archive(operation)
         elif package.source_type == "directory":
@@ -541,14 +587,11 @@ class Executor:
 
         operation_message = self.get_operation_message(operation)
         message = (
-            f"  <fg=blue;options=bold>•</> {operation_message}:"
+            f"  <fg=blue;options=bold>-</> {operation_message}:"
             " <info>Installing...</info>"
         )
         self._write(operation, message)
 
-        if not self._use_modern_installation:
-            return self.pip_install(archive, upgrade=operation.job_type == "update")
-
         try:
             if operation.job_type == "update":
                 # Uninstall first
@@ -582,12 +625,14 @@ class Executor:
 
             raise
 
-    def _prepare_archive(self, operation: Install | Update) -> Path:
+    def _prepare_archive(
+        self, operation: Install | Update, *, output_dir: Path | None = None
+    ) -> Path:
         package = operation.package
         operation_message = self.get_operation_message(operation)
 
         message = (
-            f"  <fg=blue;options=bold>•</> {operation_message}:"
+            f"  <fg=blue;options=bold>-</> {operation_message}:"
             " <info>Preparing...</info>"
         )
         self._write(operation, message)
@@ -601,20 +646,36 @@ class Executor:
 
         self._populate_hashes_dict(archive, package)
 
-        return self._chef.prepare(archive, editable=package.develop)
+        return self._chef.prepare(
+            archive,
+            editable=package.develop,
+            output_dir=output_dir,
+            config_settings=self._build_config_settings.get(operation.package.name),
+        )
 
     def _prepare_git_archive(self, operation: Install | Update) -> Path:
-        from poetry.vcs.git import Git
-
         package = operation.package
+        assert package.source_url is not None
+
+        if package.source_resolved_reference and not package.develop:
+            # Only cache git archives when we know precise reference hash,
+            # otherwise we might get stale archives
+            cached_archive = self._artifact_cache.get_cached_archive_for_git(
+                package.source_url,
+                package.source_resolved_reference,
+                package.source_subdirectory,
+                env=self._env,
+            )
+            if cached_archive is not None:
+                return cached_archive
+
         operation_message = self.get_operation_message(operation)
 
         message = (
-            f"  <fg=blue;options=bold>•</> {operation_message}: <info>Cloning...</info>"
+            f"  <fg=blue;options=bold>-</> {operation_message}: <info>Cloning...</info>"
         )
         self._write(operation, message)
 
-        assert package.source_url is not None
         source = Git.clone(
             url=package.source_url,
             source_root=self._env.path / "src",
@@ -625,77 +686,29 @@ class Executor:
         original_url = package.source_url
         package._source_url = str(source.path)
 
-        archive = self._prepare_archive(operation)
-
-        package._source_url = original_url
-
-        return archive
-
-    def _install_directory_without_wheel_installer(
-        self, operation: Install | Update
-    ) -> int:
-        from poetry.core.pyproject.toml import PyProjectTOML
-
-        from poetry.factory import Factory
-
-        package = operation.package
-        operation_message = self.get_operation_message(operation)
-
-        message = (
-            f"  <fg=blue;options=bold>•</> {operation_message}:"
-            " <info>Building...</info>"
-        )
-        self._write(operation, message)
-
-        assert package.source_url is not None
-        if package.root_dir:
-            req = package.root_dir / package.source_url
-        else:
-            req = Path(package.source_url).resolve(strict=False)
-
-        if package.source_subdirectory:
-            req /= package.source_subdirectory
-
-        pyproject = PyProjectTOML(os.path.join(req, "pyproject.toml"))
-
-        package_poetry = None
-        if pyproject.is_poetry_project():
-            with contextlib.suppress(RuntimeError):
-                package_poetry = Factory().create_poetry(pyproject.file.path.parent)
-
-        if package_poetry is not None:
-            # Even if there is a build system specified
-            # some versions of pip (< 19.0.0) don't understand it
-            # so we need to check the version of pip to know
-            # if we can rely on the build system
-            legacy_pip = (
-                self._env.pip_version
-                < self._env.pip_version.__class__.from_parts(19, 0, 0)
+        output_dir = None
+        if package.source_resolved_reference and not package.develop:
+            output_dir = self._artifact_cache.get_cache_directory_for_git(
+                original_url,
+                package.source_resolved_reference,
+                package.source_subdirectory,
             )
 
-            builder: Builder
-            if package.develop and not package_poetry.package.build_script:
-                from poetry.masonry.builders.editable import EditableBuilder
+        try:
+            archive = self._prepare_archive(operation, output_dir=output_dir)
+        except Exception:
+            # always reset source_url in case of an error for correct output
+            package._source_url = original_url
+            raise
+        if not package.develop:
+            package._source_url = original_url
 
-                # This is a Poetry package in editable mode
-                # we can use the EditableBuilder without going through pip
-                # to install it, unless it has a build script.
-                builder = EditableBuilder(package_poetry, self._env, NullIO())
-                builder.build()
+        if output_dir is not None and output_dir.is_dir():
+            # Mark directories with cached git packages, to distinguish from
+            # "normal" cache
+            (output_dir / ".created_from_git_dependency").touch()
 
-                return 0
-            elif legacy_pip or package_poetry.package.build_script:
-                from poetry.core.masonry.builders.sdist import SdistBuilder
-
-                # We need to rely on creating a temporary setup.py
-                # file since the version of pip does not support
-                # build-systems
-                # We also need it for non-PEP-517 packages
-                builder = SdistBuilder(package_poetry)
-                with builder.setup_py():
-                    return self.pip_install(req, upgrade=True, editable=package.develop)
-
-        return self.pip_install(req, upgrade=True, editable=package.develop)
+        return archive
 
     def _download(self, operation: Install | Update) -> Path:
         link = self._chooser.choose_for(operation.package)
@@ -717,38 +730,41 @@ class Executor:
     def _download_link(self, operation: Install | Update, link: Link) -> Path:
         package = operation.package
 
-        output_dir = self._chef.get_cache_directory_for_link(link)
-        # Try to get cached original package for the link provided
-        original_archive = self._chef.get_cached_archive_for_link(link, strict=True)
-        if original_archive is None:
-            # No cached original distributions was found, so we download and prepare it
-            try:
-                original_archive = self._download_archive(operation, link)
-            except BaseException:
-                cache_directory = self._chef.get_cache_directory_for_link(link)
-                cached_file = cache_directory.joinpath(link.filename)
-                # We can't use unlink(missing_ok=True) because it's not available
-                # prior to Python 3.8
-                if cached_file.exists():
-                    cached_file.unlink()
-
-                raise
+        # Get original package for the link provided
+        download_func = functools.partial(self._download_archive, operation)
+        original_archive = self._artifact_cache.get_cached_archive_for_link(
+            link, strict=True, download_func=download_func
+        )
 
         # Get potential higher prioritized cached archive, otherwise it will fall back
         # to the original archive.
-        archive = self._chef.get_cached_archive_for_link(link, strict=False)
-        # 'archive' can at this point never be None. Since we previously downloaded
-        # an archive, we now should have something cached that we can use here
-        assert archive is not None
+        archive = self._artifact_cache.get_cached_archive_for_link(
+            link,
+            strict=False,
+            env=self._env,
+        )
+        if archive is None:
+            # Since we previously downloaded an archive, we now should have
+            # something cached that we can use here. The only case in which
+            # archive is None is if the original archive is not valid for the
+            # current environment.
+            raise RuntimeError(
+                f"Package {link.url} cannot be installed in the current environment"
+                f" {self._env.marker_env}"
+            )
 
         if archive.suffix != ".whl":
             message = (
-                f"  <fg=blue;options=bold>•</> {self.get_operation_message(operation)}:"
+                f"  <fg=blue;options=bold>-</> {self.get_operation_message(operation)}:"
                 " <info>Preparing...</info>"
             )
             self._write(operation, message)
 
-            archive = self._chef.prepare(archive, output_dir=output_dir)
+            archive = self._chef.prepare(
+                archive,
+                output_dir=original_archive.parent,
+                config_settings=self._build_config_settings.get(operation.package.name),
+            )
 
         # Use the original archive to provide the correct hash.
         self._populate_hashes_dict(original_archive, package)
@@ -762,8 +778,17 @@ class Executor:
 
     @staticmethod
     def _validate_archive_hash(archive: Path, package: Package) -> str:
-        archive_hash: str = "sha256:" + get_file_hash(archive)
         known_hashes = {f["hash"] for f in package.files if f["file"] == archive.name}
+        hash_types = {t.split(":")[0] for t in known_hashes}
+        hash_type = get_highest_priority_hash_type(hash_types, archive.name)
+
+        if hash_type is None:
+            raise RuntimeError(
+                f"No usable hash type(s) for {package} from archive"
+                f" {archive.name} found (known hashes: {known_hashes!s})"
+            )
+
+        archive_hash = f"{hash_type}:{get_file_hash(archive, hash_type)}"
 
         if archive_hash not in known_hashes:
             raise RuntimeError(
@@ -773,14 +798,20 @@ class Executor:
 
         return archive_hash
 
-    def _download_archive(self, operation: Install | Update, link: Link) -> Path:
-        response = self._authenticator.request(
-            "get", link.url, stream=True, io=self._sections.get(id(operation), self._io)
+    def _download_archive(
+        self,
+        operation: Install | Update,
+        url: str,
+        dest: Path,
+    ) -> None:
+        downloader = Downloader(
+            url, dest, self._authenticator, max_retries=self._max_retries
         )
-        wheel_size = response.headers.get("content-length")
+        wheel_size = downloader.total_size
+
         operation_message = self.get_operation_message(operation)
         message = (
-            f"  <fg=blue;options=bold>•</> {operation_message}: <info>Downloading...</>"
+            f"  <fg=blue;options=bold>-</> {operation_message}: <info>Downloading...</>"
         )
         progress = None
         if self.supports_fancy_output():
@@ -799,28 +830,15 @@ class Executor:
                 self._sections[id(operation)].clear()
                 progress.start()
 
-        done = 0
-        archive = self._chef.get_cache_directory_for_link(link) / link.filename
-        archive.parent.mkdir(parents=True, exist_ok=True)
-        with atomic_open(archive) as f:
-            for chunk in response.iter_content(chunk_size=4096):
-                if not chunk:
-                    break
-
-                done += len(chunk)
-
-                if progress:
-                    with self._lock:
-                        progress.set_progress(done)
-
-                f.write(chunk)
+        for fetched_size in downloader.download_with_progress(chunk_size=4096):
+            if progress:
+                with self._lock:
+                    progress.set_progress(fetched_size)
 
         if progress:
             with self._lock:
                 progress.finish()
 
-        return archive
-
     def _should_write_operation(self, operation: Operation) -> bool:
         return (
             not operation.skipped or self._dry_run or self._verbose or not self._enabled
@@ -836,29 +854,16 @@ class Executor:
         package = operation.package
 
         if not package.source_url or package.source_type == "legacy":
-            # Since we are installing from our own distribution cache
-            # pip will write a `direct_url.json` file pointing to the cache
-            # distribution.
-            # That's not what we want, so we remove the direct_url.json file,
-            # if it exists.
-            for (
-                direct_url_json
-            ) in self._env.site_packages.find_distribution_direct_url_json_files(
-                distribution_name=package.name, writable_only=True
-            ):
-                # We can't use unlink(missing_ok=True) because it's not always available
-                if direct_url_json.exists():
-                    direct_url_json.unlink()
             return
 
         url_reference: dict[str, Any] | None = None
 
-        if package.source_type == "git":
+        if package.source_type == "git" and not package.develop:
             url_reference = self._create_git_url_reference(package)
+        elif package.source_type in ("directory", "git"):
+            url_reference = self._create_directory_url_reference(package)
         elif package.source_type == "url":
             url_reference = self._create_url_url_reference(package)
-        elif package.source_type == "directory":
-            url_reference = self._create_directory_url_reference(package)
         elif package.source_type == "file":
             url_reference = self._create_file_url_reference(package)
 
