@@ -11,7 +11,6 @@ from subprocess import CalledProcessError
 from typing import TYPE_CHECKING
 from typing import Any
 from typing import Callable
-from urllib.parse import urlparse
 
 import pytest
 
@@ -34,19 +33,18 @@ from poetry.repositories.repository_pool import RepositoryPool
 from poetry.utils.cache import ArtifactCache
 from poetry.utils.env import MockEnv
 from poetry.vcs.git.backend import Git
-from tests.repositories.test_pypi_repository import MockRepository
 
 
 if TYPE_CHECKING:
     from collections.abc import Iterator
+    from collections.abc import Mapping
+    from collections.abc import Sequence
 
-    import httpretty
-
-    from httpretty.core import HTTPrettyRequest
     from pytest_mock import MockerFixture
 
     from poetry.config.config import Config
     from poetry.installation.operations.operation import Operation
+    from poetry.repositories.pypi_repository import PyPiRepository
     from poetry.utils.env import VirtualEnv
     from tests.types import FixtureDirGetter
 
@@ -67,7 +65,12 @@ class Chef(BaseChef):
 
         self._sdist_wheels = wheels
 
-    def _prepare_sdist(self, archive: Path, destination: Path | None = None) -> Path:
+    def _prepare_sdist(
+        self,
+        archive: Path,
+        destination: Path | None = None,
+        config_settings: Mapping[str, str | Sequence[str]] | None = None,
+    ) -> Path:
         if self._sdist_wheels is not None:
             wheel = self._sdist_wheels.pop(0)
             self._sdist_wheels.append(wheel)
@@ -77,7 +80,12 @@ class Chef(BaseChef):
         return super()._prepare_sdist(archive)
 
     def _prepare(
-        self, directory: Path, destination: Path, *, editable: bool = False
+        self,
+        directory: Path,
+        destination: Path,
+        *,
+        editable: bool = False,
+        config_settings: Mapping[str, str | Sequence[str]] | None = None,
     ) -> Path:
         if self._directory_wheels is not None:
             wheel = self._directory_wheels.pop(0)
@@ -127,43 +135,15 @@ def io_not_decorated() -> BufferedIO:
 
 
 @pytest.fixture
-def pool() -> RepositoryPool:
+def pool(pypi_repository: PyPiRepository) -> RepositoryPool:
     pool = RepositoryPool()
-    pool.add_repository(MockRepository())
+
+    pypi_repository._fallback = True
+    pool.add_repository(pypi_repository)
 
     return pool
 
 
-@pytest.fixture
-def mock_file_downloads(
-    http: type[httpretty.httpretty], fixture_dir: FixtureDirGetter
-) -> None:
-    def callback(
-        request: HTTPrettyRequest, uri: str, headers: dict[str, Any]
-    ) -> list[int | dict[str, Any] | bytes]:
-        name = Path(urlparse(uri).path).name
-
-        fixture = Path(__file__).parent.parent.joinpath(
-            "repositories/fixtures/pypi.org/dists/" + name
-        )
-
-        if not fixture.exists():
-            fixture = fixture_dir("distributions") / name
-
-            if not fixture.exists():
-                fixture = (
-                    fixture_dir("distributions") / "demo-0.1.0-py2.py3-none-any.whl"
-                )
-
-        return [200, headers, fixture.read_bytes()]
-
-    http.register_uri(
-        http.GET,
-        re.compile("^https://files.pythonhosted.org/.*$"),
-        body=callback,
-    )
-
-
 @pytest.fixture
 def copy_wheel(tmp_path: Path, fixture_dir: FixtureDirGetter) -> Callable[[], Path]:
     def _copy_wheel() -> Path:
@@ -195,7 +175,6 @@ def test_execute_executes_a_batch_of_operations(
     pool: RepositoryPool,
     io: BufferedIO,
     tmp_path: Path,
-    mock_file_downloads: None,
     env: MockEnv,
     copy_wheel: Callable[[], Path],
     fixture_dir: FixtureDirGetter,
@@ -256,13 +235,13 @@ def test_execute_executes_a_batch_of_operations(
     expected = f"""
 Package operations: 4 installs, 2 updates, 1 removal
 
-  • Installing pytest (3.5.1)
-  • Removing attrs (17.4.0)
-  • Updating requests (2.18.3 -> 2.18.4)
-  • Downgrading pytest (3.5.1 -> 3.5.0)
-  • Installing demo (0.1.0 {file_package.source_url})
-  • Installing simple-project (1.2.3 {directory_package.source_url})
-  • Installing demo (0.1.0 master)
+  - Installing pytest (3.5.1)
+  - Removing attrs (17.4.0)
+  - Updating requests (2.18.3 -> 2.18.4)
+  - Downgrading pytest (3.5.1 -> 3.5.0)
+  - Installing demo (0.1.0 {file_package.source_url})
+  - Installing simple-project (1.2.3 {directory_package.source_url})
+  - Installing demo (0.1.0 master)
 """
 
     expected_lines = set(expected.splitlines())
@@ -275,8 +254,105 @@ Package operations: 4 installs, 2 updates, 1 removal
 
     assert prepare_spy.call_count == 2
     assert prepare_spy.call_args_list == [
-        mocker.call(chef, mocker.ANY, destination=mocker.ANY, editable=False),
-        mocker.call(chef, mocker.ANY, destination=mocker.ANY, editable=True),
+        mocker.call(
+            chef,
+            mocker.ANY,
+            destination=mocker.ANY,
+            editable=False,
+            config_settings=None,
+        ),
+        mocker.call(
+            chef,
+            mocker.ANY,
+            destination=mocker.ANY,
+            editable=True,
+            config_settings=None,
+        ),
+    ]
+
+
+def test_execute_build_config_settings_passed(
+    mocker: MockerFixture,
+    config: Config,
+    pool: RepositoryPool,
+    io: BufferedIO,
+    tmp_path: Path,
+    env: MockEnv,
+    copy_wheel: Callable[[], Path],
+    fixture_dir: FixtureDirGetter,
+) -> None:
+    wheel_install = mocker.patch.object(WheelInstaller, "install")
+
+    config_settings_demo = {"CC": "gcc", "--build-option": ["--one", "--two"]}
+
+    config.merge(
+        {
+            "cache-dir": str(tmp_path),
+            "installer": {"build-config-settings": {"demo": config_settings_demo}},
+        }
+    )
+    artifact_cache = ArtifactCache(cache_dir=config.artifacts_cache_directory)
+
+    prepare_spy = mocker.spy(Chef, "_prepare")
+    chef = Chef(artifact_cache, env, Factory.create_pool(config))
+    chef.set_directory_wheel([copy_wheel(), copy_wheel()])
+    chef.set_sdist_wheel(copy_wheel())
+
+    executor = Executor(env, pool, config, io)
+    executor._chef = chef
+
+    directory_package = Package(
+        "simple-project",
+        "1.2.3",
+        source_type="directory",
+        source_url=fixture_dir("simple_project").resolve().as_posix(),
+    )
+
+    git_package = Package(
+        "demo",
+        "0.1.0",
+        source_type="git",
+        source_reference="master",
+        source_url="https://github.com/demo/demo.git",
+        develop=True,
+    )
+
+    return_code = executor.execute(
+        [
+            Install(directory_package),
+            Install(git_package),
+        ]
+    )
+
+    expected = f"""
+Package operations: 2 installs, 0 updates, 0 removals
+
+  - Installing simple-project (1.2.3 {directory_package.source_url})
+  - Installing demo (0.1.0 master)
+"""
+
+    expected_lines = set(expected.splitlines())
+    output_lines = set(io.fetch_output().splitlines())
+    assert output_lines == expected_lines
+    assert wheel_install.call_count == 2
+    assert return_code == 0
+
+    assert prepare_spy.call_count == 2
+    assert prepare_spy.call_args_list == [
+        mocker.call(
+            chef,
+            mocker.ANY,
+            destination=mocker.ANY,
+            editable=False,
+            config_settings=None,
+        ),
+        mocker.call(
+            chef,
+            mocker.ANY,
+            destination=mocker.ANY,
+            editable=True,
+            config_settings=config_settings_demo,
+        ),
     ]
 
 
@@ -308,7 +384,6 @@ def test_execute_prints_warning_for_yanked_package(
     pool: RepositoryPool,
     io: BufferedIO,
     tmp_path: Path,
-    mock_file_downloads: None,
     env: MockEnv,
     operations: list[Operation],
     has_warning: bool,
@@ -342,7 +417,6 @@ def test_execute_prints_warning_for_invalid_wheels(
     pool: RepositoryPool,
     io: BufferedIO,
     tmp_path: Path,
-    mock_file_downloads: None,
     env: MockEnv,
 ) -> None:
     config.merge({"cache-dir": str(tmp_path)})
@@ -419,7 +493,7 @@ def test_execute_shows_skipped_operations_if_verbose(
     expected = """
 Package operations: 0 installs, 0 updates, 0 removals, 1 skipped
 
-  • Removing clikit (0.2.3): Skipped for the following reason: Not currently installed
+  - Removing clikit (0.2.3): Skipped for the following reason: Not currently installed
 """
     assert io.fetch_output() == expected
     assert len(env.executed) == 0
@@ -442,7 +516,7 @@ def test_execute_should_show_errors(
     expected = """
 Package operations: 1 install, 0 updates, 0 removals
 
-  • Installing clikit (0.2.3)
+  - Installing clikit (0.2.3)
 
   Exception
 
@@ -457,7 +531,6 @@ def test_execute_works_with_ansi_output(
     pool: RepositoryPool,
     io_decorated: BufferedIO,
     tmp_path: Path,
-    mock_file_downloads: None,
     env: MockEnv,
 ) -> None:
     config.merge({"cache-dir": str(tmp_path)})
@@ -473,10 +546,10 @@ def test_execute_works_with_ansi_output(
     # fmt: off
     expected = [
         "\x1b[39;1mPackage operations\x1b[39;22m: \x1b[34m1\x1b[39m install, \x1b[34m0\x1b[39m updates, \x1b[34m0\x1b[39m removals",
-        "\x1b[34;1m•\x1b[39;22m \x1b[39mInstalling \x1b[39m\x1b[36mcleo\x1b[39m\x1b[39m (\x1b[39m\x1b[39;1m1.0.0a5\x1b[39;22m\x1b[39m)\x1b[39m: \x1b[34mPending...\x1b[39m",
-        "\x1b[34;1m•\x1b[39;22m \x1b[39mInstalling \x1b[39m\x1b[36mcleo\x1b[39m\x1b[39m (\x1b[39m\x1b[39;1m1.0.0a5\x1b[39;22m\x1b[39m)\x1b[39m: \x1b[34mDownloading...\x1b[39m",
-        "\x1b[34;1m•\x1b[39;22m \x1b[39mInstalling \x1b[39m\x1b[36mcleo\x1b[39m\x1b[39m (\x1b[39m\x1b[39;1m1.0.0a5\x1b[39;22m\x1b[39m)\x1b[39m: \x1b[34mInstalling...\x1b[39m",
-        "\x1b[32;1m•\x1b[39;22m \x1b[39mInstalling \x1b[39m\x1b[36mcleo\x1b[39m\x1b[39m (\x1b[39m\x1b[32m1.0.0a5\x1b[39m\x1b[39m)\x1b[39m",  # finished
+        "\x1b[34;1m-\x1b[39;22m \x1b[39mInstalling \x1b[39m\x1b[36mcleo\x1b[39m\x1b[39m (\x1b[39m\x1b[39;1m1.0.0a5\x1b[39;22m\x1b[39m)\x1b[39m: \x1b[34mPending...\x1b[39m",
+        "\x1b[34;1m-\x1b[39;22m \x1b[39mInstalling \x1b[39m\x1b[36mcleo\x1b[39m\x1b[39m (\x1b[39m\x1b[39;1m1.0.0a5\x1b[39;22m\x1b[39m)\x1b[39m: \x1b[34mDownloading...\x1b[39m",
+        "\x1b[34;1m-\x1b[39;22m \x1b[39mInstalling \x1b[39m\x1b[36mcleo\x1b[39m\x1b[39m (\x1b[39m\x1b[39;1m1.0.0a5\x1b[39;22m\x1b[39m)\x1b[39m: \x1b[34mInstalling...\x1b[39m",
+        "\x1b[32;1m-\x1b[39;22m \x1b[39mInstalling \x1b[39m\x1b[36mcleo\x1b[39m\x1b[39m (\x1b[39m\x1b[32m1.0.0a5\x1b[39m\x1b[39m)\x1b[39m",  # finished
     ]
     # fmt: on
 
@@ -494,7 +567,6 @@ def test_execute_works_with_no_ansi_output(
     pool: RepositoryPool,
     io_not_decorated: BufferedIO,
     tmp_path: Path,
-    mock_file_downloads: None,
     env: MockEnv,
 ) -> None:
     config.merge({"cache-dir": str(tmp_path)})
@@ -510,7 +582,7 @@ def test_execute_works_with_no_ansi_output(
     expected = """
 Package operations: 1 install, 0 updates, 0 removals
 
-  • Installing cleo (1.0.0a5)
+  - Installing cleo (1.0.0a5)
 """
     expected_lines = set(expected.splitlines())
     output_lines = set(io_not_decorated.fetch_output().splitlines())
@@ -536,8 +608,8 @@ def test_execute_should_show_operation_as_cancelled_on_subprocess_keyboard_inter
     expected = """
 Package operations: 1 install, 0 updates, 0 removals
 
-  • Installing clikit (0.2.3)
-  • Installing clikit (0.2.3): Cancelled
+  - Installing clikit (0.2.3)
+  - Installing clikit (0.2.3): Cancelled
 """
 
     assert io.fetch_output() == expected
@@ -557,6 +629,7 @@ def test_execute_should_gracefully_handle_io_error(
 
     def write_line(string: str, **kwargs: Any) -> None:
         # Simulate UnicodeEncodeError
+        string = string.replace("-", "•")
         string.encode("ascii")
         original_write_line(string, **kwargs)
 
@@ -580,7 +653,6 @@ def test_executor_should_delete_incomplete_downloads(
     tmp_path: Path,
     mocker: MockerFixture,
     pool: RepositoryPool,
-    mock_file_downloads: None,
     env: MockEnv,
 ) -> None:
     cached_archive = tmp_path / "tomlkit-0.5.3-py2.py3-none-any.whl"
@@ -670,7 +742,9 @@ def test_executor_should_not_write_pep610_url_references_for_cached_package(
     package.files = [
         {
             "file": "demo-0.1.0-py2.py3-none-any.whl",
-            "hash": "sha256:70e704135718fffbcbf61ed1fc45933cfd86951a744b681000eaaa75da31f17a",
+            "hash": (
+                "sha256:70e704135718fffbcbf61ed1fc45933cfd86951a744b681000eaaa75da31f17a"
+            ),
         }
     ]
 
@@ -697,7 +771,9 @@ def test_executor_should_write_pep610_url_references_for_wheel_files(
     package.files = [
         {
             "file": "demo-0.1.0-py2.py3-none-any.whl",
-            "hash": "sha256:70e704135718fffbcbf61ed1fc45933cfd86951a744b681000eaaa75da31f17a",
+            "hash": (
+                "sha256:70e704135718fffbcbf61ed1fc45933cfd86951a744b681000eaaa75da31f17a"
+            ),
         }
     ]
 
@@ -730,7 +806,9 @@ def test_executor_should_write_pep610_url_references_for_non_wheel_files(
     package.files = [
         {
             "file": "demo-0.1.0.tar.gz",
-            "hash": "sha256:9fa123ad707a5c6c944743bf3e11a0e80d86cb518d3cf25320866ca3ef43e2ad",
+            "hash": (
+                "sha256:9fa123ad707a5c6c944743bf3e11a0e80d86cb518d3cf25320866ca3ef43e2ad"
+            ),
         }
     ]
 
@@ -816,7 +894,6 @@ def test_executor_should_write_pep610_url_references_for_wheel_urls(
     pool: RepositoryPool,
     config: Config,
     io: BufferedIO,
-    mock_file_downloads: None,
     mocker: MockerFixture,
     fixture_dir: FixtureDirGetter,
     is_artifact_cached: bool,
@@ -839,7 +916,9 @@ def test_executor_should_write_pep610_url_references_for_wheel_urls(
     package.files = [
         {
             "file": "demo-0.1.0-py2.py3-none-any.whl",
-            "hash": "sha256:70e704135718fffbcbf61ed1fc45933cfd86951a744b681000eaaa75da31f17a",
+            "hash": (
+                "sha256:70e704135718fffbcbf61ed1fc45933cfd86951a744b681000eaaa75da31f17a"
+            ),
         }
     ]
 
@@ -890,7 +969,6 @@ def test_executor_should_write_pep610_url_references_for_non_wheel_urls(
     pool: RepositoryPool,
     config: Config,
     io: BufferedIO,
-    mock_file_downloads: None,
     mocker: MockerFixture,
     fixture_dir: FixtureDirGetter,
     is_sdist_cached: bool,
@@ -933,7 +1011,9 @@ def test_executor_should_write_pep610_url_references_for_non_wheel_urls(
     package.files = [
         {
             "file": "demo-0.1.0.tar.gz",
-            "hash": "sha256:9fa123ad707a5c6c944743bf3e11a0e80d86cb518d3cf25320866ca3ef43e2ad",
+            "hash": (
+                "sha256:9fa123ad707a5c6c944743bf3e11a0e80d86cb518d3cf25320866ca3ef43e2ad"
+            ),
         }
     ]
 
@@ -968,6 +1048,13 @@ def test_executor_should_write_pep610_url_references_for_non_wheel_urls(
         download_spy.assert_not_called()
 
 
+@pytest.mark.parametrize(
+    "source_url,written_source_url",
+    [
+        ("https://github.com/demo/demo.git", "https://github.com/demo/demo.git"),
+        ("git@github.com:demo/demo.git", "ssh://git@github.com/demo/demo.git"),
+    ],
+)
 @pytest.mark.parametrize("is_artifact_cached", [False, True])
 def test_executor_should_write_pep610_url_references_for_git(
     tmp_venv: VirtualEnv,
@@ -975,10 +1062,11 @@ def test_executor_should_write_pep610_url_references_for_git(
     config: Config,
     artifact_cache: ArtifactCache,
     io: BufferedIO,
-    mock_file_downloads: None,
     wheel: Path,
     mocker: MockerFixture,
     fixture_dir: FixtureDirGetter,
+    source_url: str,
+    written_source_url: str,
     is_artifact_cached: bool,
 ) -> None:
     if is_artifact_cached:
@@ -990,7 +1078,7 @@ def test_executor_should_write_pep610_url_references_for_git(
     clone_spy = mocker.spy(Git, "clone")
 
     source_resolved_reference = "123456"
-    source_url = "https://github.com/demo/demo.git"
+    source_url = source_url
 
     package = Package(
         "demo",
@@ -1001,6 +1089,8 @@ def test_executor_should_write_pep610_url_references_for_git(
         source_url=source_url,
     )
 
+    assert package.source_url == written_source_url
+
     chef = Chef(artifact_cache, tmp_venv, Factory.create_pool(config))
     chef.set_directory_wheel(wheel)
     prepare_spy = mocker.spy(chef, "prepare")
@@ -1026,7 +1116,9 @@ def test_executor_should_write_pep610_url_references_for_git(
         prepare_spy.assert_not_called()
     else:
         clone_spy.assert_called_once_with(
-            url=source_url, source_root=mocker.ANY, revision=source_resolved_reference
+            url=package.source_url,
+            source_root=mocker.ANY,
+            revision=source_resolved_reference,
         )
         prepare_spy.assert_called_once()
         assert prepare_spy.spy_return.exists(), "cached file should not be deleted"
@@ -1039,7 +1131,6 @@ def test_executor_should_write_pep610_url_references_for_editable_git(
     config: Config,
     artifact_cache: ArtifactCache,
     io: BufferedIO,
-    mock_file_downloads: None,
     wheel: Path,
     mocker: MockerFixture,
     fixture_dir: FixtureDirGetter,
@@ -1088,7 +1179,6 @@ def test_executor_should_append_subdirectory_for_git(
     config: Config,
     artifact_cache: ArtifactCache,
     io: BufferedIO,
-    mock_file_downloads: None,
     wheel: Path,
 ) -> None:
     package = Package(
@@ -1110,7 +1200,92 @@ def test_executor_should_append_subdirectory_for_git(
     executor.execute([Install(package)])
 
     archive_arg = spy.call_args[0][0]
-    assert archive_arg == tmp_venv.path / "src/demo/subdirectories/two"
+    assert archive_arg == tmp_venv.path / "src/subdirectories/two"
+
+
+def test_executor_should_install_multiple_packages_from_same_git_repository(
+    mocker: MockerFixture,
+    tmp_venv: VirtualEnv,
+    pool: RepositoryPool,
+    config: Config,
+    artifact_cache: ArtifactCache,
+    io: BufferedIO,
+    wheel: Path,
+) -> None:
+    package_a = Package(
+        "package_a",
+        "0.1.2",
+        source_type="git",
+        source_reference="master",
+        source_resolved_reference="123456",
+        source_url="https://github.com/demo/subdirectories.git",
+        source_subdirectory="package_a",
+    )
+    package_b = Package(
+        "package_b",
+        "0.1.2",
+        source_type="git",
+        source_reference="master",
+        source_resolved_reference="123456",
+        source_url="https://github.com/demo/subdirectories.git",
+        source_subdirectory="package_b",
+    )
+
+    chef = Chef(artifact_cache, tmp_venv, Factory.create_pool(config))
+    chef.set_directory_wheel(wheel)
+    spy = mocker.spy(chef, "prepare")
+
+    executor = Executor(tmp_venv, pool, config, io)
+    executor._chef = chef
+    executor.execute([Install(package_a), Install(package_b)])
+
+    archive_arg = spy.call_args_list[0][0][0]
+    assert archive_arg == tmp_venv.path / "src/subdirectories/package_a"
+
+    archive_arg = spy.call_args_list[1][0][0]
+    assert archive_arg == tmp_venv.path / "src/subdirectories/package_b"
+
+
+def test_executor_should_install_multiple_packages_from_forked_git_repository(
+    mocker: MockerFixture,
+    tmp_venv: VirtualEnv,
+    pool: RepositoryPool,
+    config: Config,
+    artifact_cache: ArtifactCache,
+    io: BufferedIO,
+    wheel: Path,
+) -> None:
+    package_a = Package(
+        "one",
+        "1.0.0",
+        source_type="git",
+        source_reference="master",
+        source_resolved_reference="123456",
+        source_url="https://github.com/demo/subdirectories.git",
+        source_subdirectory="one",
+    )
+    package_b = Package(
+        "two",
+        "2.0.0",
+        source_type="git",
+        source_reference="master",
+        source_resolved_reference="123456",
+        source_url="https://github.com/forked_demo/subdirectories.git",
+        source_subdirectory="two",
+    )
+
+    chef = Chef(artifact_cache, tmp_venv, Factory.create_pool(config))
+    chef.set_directory_wheel(wheel)
+    prepare_spy = mocker.spy(chef, "prepare")
+
+    executor = Executor(tmp_venv, pool, config, io)
+    executor._chef = chef
+    executor.execute([Install(package_a), Install(package_b)])
+
+    # Verify that the repo for package_a is not re-used for package_b.
+    # both repos must be cloned serially into separate directories.
+    # If so, executor.prepare() will be called twice.
+    assert prepare_spy.call_count == 2
 
 
 def test_executor_should_write_pep610_url_references_for_git_with_subdirectories(
@@ -1119,7 +1294,6 @@ def test_executor_should_write_pep610_url_references_for_git_with_subdirectories
     config: Config,
     artifact_cache: ArtifactCache,
     io: BufferedIO,
-    mock_file_downloads: None,
     wheel: Path,
 ) -> None:
     package = Package(
@@ -1184,65 +1358,6 @@ def test_executor_should_be_initialized_with_correct_workers(
     assert executor._max_workers == expected_workers
 
 
-def test_executor_fallback_on_poetry_create_error_without_wheel_installer(
-    mocker: MockerFixture,
-    config: Config,
-    pool: RepositoryPool,
-    io: BufferedIO,
-    tmp_path: Path,
-    mock_file_downloads: None,
-    env: MockEnv,
-    fixture_dir: FixtureDirGetter,
-) -> None:
-    mock_pip_install = mocker.patch("poetry.installation.executor.pip_install")
-    mock_sdist_builder = mocker.patch("poetry.core.masonry.builders.sdist.SdistBuilder")
-    mock_editable_builder = mocker.patch(
-        "poetry.masonry.builders.editable.EditableBuilder"
-    )
-    mock_create_poetry = mocker.patch(
-        "poetry.factory.Factory.create_poetry", side_effect=RuntimeError
-    )
-
-    config.merge(
-        {
-            "cache-dir": str(tmp_path),
-            "installer": {"modern-installation": False},
-        }
-    )
-
-    executor = Executor(env, pool, config, io)
-
-    directory_package = Package(
-        "simple-project",
-        "1.2.3",
-        source_type="directory",
-        source_url=fixture_dir("simple_project").resolve().as_posix(),
-    )
-
-    return_code = executor.execute(
-        [
-            Install(directory_package),
-        ]
-    )
-
-    expected = f"""
-Package operations: 1 install, 0 updates, 0 removals
-
-  • Installing simple-project (1.2.3 {directory_package.source_url})
-"""
-
-    expected_lines = set(expected.splitlines())
-    output_lines = set(io.fetch_output().splitlines())
-    assert output_lines == expected_lines
-    assert return_code == 0
-    assert mock_create_poetry.call_count == 1
-    assert mock_sdist_builder.call_count == 0
-    assert mock_editable_builder.call_count == 0
-    assert mock_pip_install.call_count == 1
-    assert mock_pip_install.call_args[1].get("upgrade") is True
-    assert mock_pip_install.call_args[1].get("editable") is False
-
-
 @pytest.mark.parametrize("failing_method", ["build", "get_requires_for_build"])
 @pytest.mark.parametrize(
     "exception",
@@ -1252,15 +1367,16 @@ Package operations: 1 install, 0 updates, 0 removals
     ],
 )
 @pytest.mark.parametrize("editable", [False, True])
+@pytest.mark.parametrize("source_type", ["directory", "git", "git subdirectory"])
 def test_build_backend_errors_are_reported_correctly_if_caused_by_subprocess(
     failing_method: str,
     exception: Exception,
     editable: bool,
+    source_type: str,
     mocker: MockerFixture,
     config: Config,
     pool: RepositoryPool,
     io: BufferedIO,
-    mock_file_downloads: None,
     env: MockEnv,
     fixture_dir: FixtureDirGetter,
 ) -> None:
@@ -1272,52 +1388,95 @@ def test_build_backend_errors_are_reported_correctly_if_caused_by_subprocess(
 
     package_name = "simple-project"
     package_version = "1.2.3"
-    directory_package = Package(
+    source_reference: str | None = None
+    source_sub_directory: str | None = None
+    if source_type == "directory":
+        source_url = fixture_dir("simple_project").resolve().as_posix()
+        source_resolved_reference = None
+        pip_url = path_to_url(source_url)
+        pip_editable_requirement = source_url
+    elif source_type == "git":
+        source_url = "https://github.com/demo/demo.git"
+        source_reference = "v2.0"
+        source_resolved_reference = "12345678"
+        pip_url = f"git+{source_url}@{source_reference}"
+        pip_editable_requirement = f"{pip_url}#egg={package_name}"
+    elif source_type == "git subdirectory":
+        source_type = "git"
+        source_sub_directory = "one"
+        source_url = "https://github.com/demo/subdirectories.git"
+        source_reference = "v2.0"
+        source_resolved_reference = "12345678"
+        pip_base_url = f"git+{source_url}@{source_reference}"
+        pip_url = f"{pip_base_url}#subdirectory={source_sub_directory}"
+        pip_editable_requirement = (
+            f"{pip_base_url}#egg={package_name}&subdirectory={source_sub_directory}"
+        )
+    else:
+        raise ValueError(f"Unknown source type: {source_type}")
+    package = Package(
         package_name,
         package_version,
-        source_type="directory",
-        source_url=fixture_dir("simple_project").resolve().as_posix(),
+        source_type=source_type,
+        source_url=source_url,
+        source_reference=source_reference,
+        source_resolved_reference=source_resolved_reference,
+        source_subdirectory=source_sub_directory,
         develop=editable,
     )
     # must not be included in the error message
-    directory_package.python_versions = ">=3.7"
+    package.python_versions = ">=3.7"
 
-    return_code = executor.execute([Install(directory_package)])
+    return_code = executor.execute([Install(package)])
 
     assert return_code == 1
 
-    package_url = directory_package.source_url
-    expected_start = f"""
-Package operations: 1 install, 0 updates, 0 removals
-
-  • Installing {package_name} ({package_version} {package_url})
-
-  ChefBuildError
-
-  hide the original error
-  \
-
-  original error
-"""
-
-    assert directory_package.source_url is not None
+    assert package.source_url is not None
     if editable:
         pip_command = "pip wheel --no-cache-dir --use-pep517 --editable"
-        requirement = directory_package.source_url
-        assert Path(requirement).exists()
+        requirement = pip_editable_requirement
+        if source_type == "directory":
+            assert Path(requirement).exists()
     else:
         pip_command = "pip wheel --no-cache-dir --use-pep517"
-        requirement = f"{package_name} @ {path_to_url(directory_package.source_url)}"
-    expected_end = f"""
-Note: This error originates from the build backend, and is likely not a problem with \
-poetry but with {package_name} ({package_version} {package_url}) not supporting \
-PEP 517 builds. You can verify this by running '{pip_command} "{requirement}"'.
+        requirement = f"{package_name} @ {pip_url}"
+
+    version_details = package.source_resolved_reference or package.source_url
+    expected_source_string = f"{package_name} ({package_version} {version_details})"
+    expected_pip_command = f'{pip_command} "{requirement}"'
+
+    expected_output = f"""
+Package operations: 1 install, 0 updates, 0 removals
+
+  - Installing {expected_source_string}
+
+PEP517 build of a dependency failed
+
+hide the original error
+"""
+
+    if isinstance(exception, CalledProcessError):
+        expected_output += (
+            "\n    | Command '['pip']' returned non-zero exit status 1."
+            "\n    | "
+            "\n    | original error"
+            "\n"
+        )
+
+    expected_output += f"""
+Note: This error originates from the build backend, and is likely not a problem \
+with poetry but one of the following issues with {expected_source_string}
+
+  - not supporting PEP 517 builds
+  - not specifying PEP 517 build requirements correctly
+  - the build requirements are incompatible with your operating system or Python version
+  - the build requirements are missing system dependencies (eg: compilers, libraries, headers).
+
+You can verify this by running {expected_pip_command}.
 
 """
 
-    output = io.fetch_output()
-    assert output.startswith(expected_start)
-    assert output.endswith(expected_end)
+    assert io.fetch_output() == expected_output
 
 
 @pytest.mark.parametrize("encoding", ["utf-8", "latin-1"])
@@ -1329,7 +1488,6 @@ def test_build_backend_errors_are_reported_correctly_if_caused_by_subprocess_enc
     config: Config,
     pool: RepositoryPool,
     io: BufferedIO,
-    mock_file_downloads: None,
     env: MockEnv,
     fixture_dir: FixtureDirGetter,
 ) -> None:
@@ -1365,7 +1523,6 @@ def test_build_system_requires_not_available(
     config: Config,
     pool: RepositoryPool,
     io: BufferedIO,
-    mock_file_downloads: None,
     env: MockEnv,
     fixture_dir: FixtureDirGetter,
 ) -> None:
@@ -1392,9 +1549,9 @@ def test_build_system_requires_not_available(
     expected_start = f"""\
 Package operations: 1 install, 0 updates, 0 removals
 
-  • Installing {package_name} ({package_version} {package_url})
+  - Installing {package_name} ({package_version} {package_url})
 
-  SolveFailure
+  SolveFailureError
 
   Because -root- depends on poetry-core (0.999) which doesn't match any versions,\
  version solving failed.
@@ -1411,7 +1568,6 @@ def test_build_system_requires_install_failure(
     config: Config,
     pool: RepositoryPool,
     io: BufferedIO,
-    mock_file_downloads: None,
     env: MockEnv,
     fixture_dir: FixtureDirGetter,
 ) -> None:
@@ -1439,9 +1595,9 @@ def test_build_system_requires_install_failure(
     expected_start = f"""\
 Package operations: 1 install, 0 updates, 0 removals
 
-  • Installing {package_name} ({package_version} {package_url})
+  - Installing {package_name} ({package_version} {package_url})
 
-  ChefInstallError
+  IsolatedBuildInstallError
 
   Failed to install poetry-core>=1.1.0a7.
   \
@@ -1458,6 +1614,7 @@ Package operations: 1 install, 0 updates, 0 removals
 
     mocker.stopall()  # to get real output
     output = io.fetch_output().strip()
+
     assert output.startswith(expected_start)
     assert output.endswith(expected_end)
 
@@ -1466,7 +1623,6 @@ def test_other_error(
     config: Config,
     pool: RepositoryPool,
     io: BufferedIO,
-    mock_file_downloads: None,
     env: MockEnv,
     fixture_dir: FixtureDirGetter,
 ) -> None:
@@ -1491,7 +1647,7 @@ def test_other_error(
     expected_start = f"""\
 Package operations: 1 install, 0 updates, 0 removals
 
-  • Installing {package_name} ({package_version} {package_url})
+  - Installing {package_name} ({package_version} {package_url})
 
   FileNotFoundError
 """
@@ -1500,3 +1656,126 @@ Package operations: 1 install, 0 updates, 0 removals
     output = io.fetch_output().strip()
     assert output.startswith(expected_start)
     assert output.endswith(expected_end)
+
+
+@pytest.mark.parametrize(
+    "package_files,expected_url_reference",
+    [
+        (
+            [
+                {
+                    "file": "demo-0.1.0.tar.gz",
+                    "hash": "sha512:766ecf369b6bdf801f6f7bbfe23923cc9793d633a55619472cd3d5763f9154711fbf57c8b6ca74e4a82fa9bd8380af831e7b8668e68e362669fc60b1d81d79ad",
+                },
+                {
+                    "file": "demo-0.1.0.tar.gz",
+                    "hash": "md5:d1912c917363a64e127318655f7d1fe7",
+                },
+                {
+                    "file": "demo-0.1.0.whl",
+                    "hash": "sha256:70e704135718fffbcbf61ed1fc45933cfd86951a744b681000eaaa75da31f17a",
+                },
+            ],
+            {
+                "archive_info": {
+                    "hashes": {
+                        "sha512": "766ecf369b6bdf801f6f7bbfe23923cc9793d633a55619472cd3d5763f9154711fbf57c8b6ca74e4a82fa9bd8380af831e7b8668e68e362669fc60b1d81d79ad"
+                    },
+                },
+            },
+        ),
+        (
+            [
+                {
+                    "file": "demo-0.1.0.tar.gz",
+                    "hash": "md5:d1912c917363a64e127318655f7d1fe7",
+                }
+            ],
+            {
+                "archive_info": {
+                    "hashes": {"md5": "d1912c917363a64e127318655f7d1fe7"},
+                },
+            },
+        ),
+        (
+            [
+                {
+                    "file": "demo-0.1.0.tar.gz",
+                    "hash": "sha3_512:196f4af9099185054ed72ca1d4c57707da5d724df0af7c3dfcc0fd018b0e0533908e790a291600c7d196fe4411b4f5f6db45213fe6e5cd5512bf18b2e9eff728",
+                },
+                {
+                    "file": "demo-0.1.0.tar.gz",
+                    "hash": "sha512:766ecf369b6bdf801f6f7bbfe23923cc9793d633a55619472cd3d5763f9154711fbf57c8b6ca74e4a82fa9bd8380af831e7b8668e68e362669fc60b1d81d79ad",
+                },
+                {
+                    "file": "demo-0.1.0.tar.gz",
+                    "hash": "md5:d1912c917363a64e127318655f7d1fe7",
+                },
+                {
+                    "file": "demo-0.1.0.whl",
+                    "hash": "sha256:70e704135718fffbcbf61ed1fc45933cfd86951a744b681000eaaa75da31f17a",
+                },
+            ],
+            {
+                "archive_info": {
+                    "hashes": {
+                        "sha3_512": "196f4af9099185054ed72ca1d4c57707da5d724df0af7c3dfcc0fd018b0e0533908e790a291600c7d196fe4411b4f5f6db45213fe6e5cd5512bf18b2e9eff728"
+                    },
+                },
+            },
+        ),
+    ],
+)
+def test_executor_known_hashes(
+    package_files: list[dict[str, str]],
+    expected_url_reference: dict[str, Any],
+    tmp_venv: VirtualEnv,
+    pool: RepositoryPool,
+    config: Config,
+    io: BufferedIO,
+    fixture_dir: FixtureDirGetter,
+) -> None:
+    package_source_url: Path = (
+        fixture_dir("distributions") / "demo-0.1.0.tar.gz"
+    ).resolve()
+    package = Package(
+        "demo", "0.1.0", source_type="file", source_url=package_source_url.as_posix()
+    )
+    package.files = package_files
+    executor = Executor(tmp_venv, pool, config, io)
+    executor.execute([Install(package)])
+    expected_url_reference["url"] = package_source_url.as_uri()
+    verify_installed_distribution(tmp_venv, package, expected_url_reference)
+
+
+def test_executor_no_supported_hash_types(
+    tmp_venv: VirtualEnv,
+    pool: RepositoryPool,
+    config: Config,
+    io: BufferedIO,
+    fixture_dir: FixtureDirGetter,
+) -> None:
+    url = (fixture_dir("distributions") / "demo-0.1.0.tar.gz").resolve()
+    package = Package("demo", "0.1.0", source_type="file", source_url=url.as_posix())
+    # Set package.files so the executor will attempt to hash the package
+    package.files = [
+        {
+            "file": "demo-0.1.0.tar.gz",
+            "hash": "hash_blah:1234567890abcdefghijklmnopqrstyzwxyz",
+        },
+        {
+            "file": "demo-0.1.0.whl",
+            "hash": "sha256:70e704135718fffbcbf61ed1fc45933cfd86951a744b681000eaaa75da31f17a",
+        },
+    ]
+
+    executor = Executor(tmp_venv, pool, config, io)
+    return_code = executor.execute([Install(package)])
+    distributions = list(tmp_venv.site_packages.distributions(name=package.name))
+    assert len(distributions) == 0
+
+    output = io.fetch_output()
+    error = io.fetch_error()
+    assert return_code == 1, f"\noutput: {output}\nerror: {error}\n"
+    assert "No usable hash type(s) for demo" in output
+    assert "hash_blah:1234567890abcdefghijklmnopqrstyzwxyz" in output
