@@ -9,8 +9,12 @@ from pathlib import Path
 from threading import Thread
 from typing import TYPE_CHECKING
 
+import packaging.tags
 import pytest
 
+from deepdiff.diff import DeepDiff
+from installer.utils import SCHEME_NAMES
+
 from poetry.factory import Factory
 from poetry.repositories.installed_repository import InstalledRepository
 from poetry.utils._compat import WINDOWS
@@ -23,6 +27,7 @@ from poetry.utils.env import SystemEnv
 from poetry.utils.env import VirtualEnv
 from poetry.utils.env import build_environment
 from poetry.utils.env import ephemeral_environment
+from poetry.utils.helpers import is_dir_writable
 
 
 if TYPE_CHECKING:
@@ -88,16 +93,27 @@ def test_env_commands_with_spaces_in_their_arg_work_as_expected(
     assert re.match(r"pip \S+ from", output)
 
 
+@pytest.mark.parametrize("differing_platform", [True, False])
 def test_env_get_supported_tags_matches_inside_virtualenv(
-    tmp_path: Path, manager: EnvManager
+    tmp_path: Path, manager: EnvManager, mocker: MockerFixture, differing_platform: bool
 ) -> None:
     venv_path = tmp_path / "Virtual Env"
     manager.build_venv(venv_path)
     venv = VirtualEnv(venv_path)
 
-    import packaging.tags
+    run_python_script_spy = mocker.spy(venv, "run_python_script")
 
-    assert venv.get_supported_tags() == list(packaging.tags.sys_tags())
+    # determine expected tags before patching sysconfig!
+    expected_tags = list(packaging.tags.sys_tags())
+
+    if differing_platform:
+        mocker.patch("sysconfig.get_platform", return_value="some_other_platform")
+        expected_call_count = 2
+    else:
+        expected_call_count = 1
+
+    assert venv.get_supported_tags() == expected_tags
+    assert run_python_script_spy.call_count == expected_call_count
 
 
 @pytest.mark.skipif(os.name == "nt", reason="Symlinks are not support for Windows")
@@ -309,7 +325,7 @@ def test_env_system_packages_are_relative_to_lib(
 
     # These are the virtual environments' base env packages,
     # in this case the system site packages.
-    for dist in metadata.distributions(path=[str(env.parent_env.site_packages.path)]):
+    for dist in env.parent_env.site_packages.distributions():
         assert (
             env.is_path_relative_to_lib(
                 Path(str(dist._path))  # type: ignore[attr-defined]
@@ -510,3 +526,37 @@ def test_command_from_bin_preserves_relative_path(manager: EnvManager) -> None:
     env = manager.get()
     command = env.get_command_from_bin("./foo.py")
     assert command == ["./foo.py"]
+
+
+@pytest.fixture
+def system_env_read_only(system_env: SystemEnv, mocker: MockerFixture) -> SystemEnv:
+    original_is_dir_writable = is_dir_writable
+
+    read_only_paths = {system_env.paths[key] for key in SCHEME_NAMES}
+
+    def mock_is_dir_writable(path: Path, create: bool = False) -> bool:
+        if str(path) in read_only_paths:
+            return False
+        return original_is_dir_writable(path, create)
+
+    mocker.patch("poetry.utils.env.base_env.is_dir_writable", new=mock_is_dir_writable)
+
+    return system_env
+
+
+def test_env_scheme_dict_returns_original_when_writable(system_env: SystemEnv) -> None:
+    assert not DeepDiff(system_env.scheme_dict, system_env.paths, ignore_order=True)
+
+
+def test_env_scheme_dict_returns_modified_when_read_only(
+    system_env_read_only: SystemEnv,
+) -> None:
+    scheme_dict = system_env_read_only.scheme_dict
+    assert DeepDiff(scheme_dict, system_env_read_only.paths, ignore_order=True)
+
+    paths = system_env_read_only.paths
+    assert all(
+        Path(scheme_dict[scheme]).exists()
+        and scheme_dict[scheme].startswith(paths["userbase"])
+        for scheme in SCHEME_NAMES
+    )
